{% extends "base.html" %}

{% block title %}IceProd Config{% end %}

{% block head_scripts %}
<link rel="stylesheet" href="/static/submit.css" media="all" type="text/css" />
{% end %}

{% block page_title %}{% if submit %}Submit{% else %}Config{% end %} Status{% end %}


{% block body %}
<div style="margin: 1rem;max-width:40rem">
    <p>Please wait while the dataset {% if submit %}submission{% else %}config update{% end %} is processed.</p>
    <img src="/static/Gray_circles_rotate.gif" style="width: 50px;margin: 1rem" />
    <p>Status: {{ status }}</p>
    {% if message %}
    <p>Message:</p>
    <div id="error">{{ message }}</div>
    {% end %}
    <p style="margin-top:5rem">If the page does not reload every second, please manually reload. This should normally take less than 10 seconds.</p>
</div>
{% end %}

{% block body_scripts %}
<script type="text/javascript">
window.location.replace('/{% if submit %}submit{% else %}config{% end %}/status/{{ message_id }}');
setTimeout(function() {
    location.reload();
}, 1000);
</script>
{% end %}
