<!doctype html>
<html lang="en">
  <head>
    <meta charset="utf-8" />
    <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1" />
    <title>{% block title %}Default title{% end %}</title>
    <meta name="description" content="{% block description %}IceProd: a grid middleware for submitting and tracking sets of jobs.{% end %}">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="shortcut icon" href="/static/favicon.ico">
    <link rel="icon" href="/static/favicon.png">
    <link rel="stylesheet" href="/static/base.css" media="all" type="text/css" />
    <link rel="stylesheet" href="/static/chosen.min.css" media="all" type="text/css" />
    {% block head_scripts %}{% end %}
  </head>
  <body>
<!--[if lt IE 9]>
<p class="browsehappy">You are using an <strong>outdated</strong> browser. Please <a href="http://browsehappy.com/">upgrade your browser</a> to improve your experience.</p>
<![endif]-->
    <div class="page">
        <header>
            <a href="/" class="title">
                <img width=155 height=60 src="/static/logo_155x60.png" alt="IceProd" /></a>
            </a>
            <nav class="menu">
                <ul>
                <li{% if section == 'dataset' %} class="current"{% end %}><a href="/dataset">Datasets</a></li>
                <!--
                <li{% if section == 'util' %} class="current"{% end %}><a href="/help">Help</a></li>
                -->
                {% if current_user %}
                  <li{% if section == 'profile' %} class="current"{% end %}><a href="/profile">Profile</a></li>
                {% end %}
                <li>
                    {% if (not current_user) %} <a href='/login?next={{ url_escape(request.uri) }}'>Login</a>
                    {% else %} <a href='/logout'>Logout</a>
                    {% end %}
                </li>
                </ul>
            </nav>
        </header>
        <div class="body {{ section.lower().replace(' ','_') }}">
            <div class="sidebar">{% block sidebar %}{% end %}</div>
            <div class="contents">
                <h1 id="page_title">{% block page_title %}{{ section }}{% end %}</h1>
                {% block body %}The Body{% end %}
            </div>
        </div>
        <footer>{% block footer %}IceProd {{ version }}{% end %}</footer>
    </div>
    <script src="//ajax.googleapis.com/ajax/libs/jquery/1.9.1/jquery.min.js"></script>
    <script src="//ajax.googleapis.com/ajax/libs/jqueryui/1.10.2/jquery-ui.min.js"></script>
    <script src="/static/util.js"></script>
    {% block body_scripts %}{% end %}
    <script src="/static/chosen.jquery.min.js"></script>
    <script>$(function() {$(".chosen-select").chosen();});</script>
  </body>
</html>
