var dataclasses={"classes":{"Batchsys":{"*":{}},"Class":{"name":"","src":"","resource_name":"","recursive":false,"libs":"","env_vars":""},"Data":{"type":["permanent",["permanent","tray_temp","task_temp","job_temp","dataset_temp","site_temp"]],"movement":["both",["input","output","both"]],"remote":"","local":"","compression":[false,[false,true,"none","gzip","gz","bzip","bz2","lzma"]],"transfer":true},"DataCenter":{"name":null,"personnel":[[],"Personnel"]},"Dif":{"entry_id":null,"entry_title":null,"parameters":"","iso_topic_category":"geoscientificinformation","data_ceter":null,"summary":"","metadata_name":"[CEOS IDN DIF]","metadata_version":"9.4","personnel":[[],"Personnel"],"sensor_name":"ICECUBE","source_name":"SIMULATION","dif_creation_date":"2021-01-22"},"DifPlus":{"dif":[null,"Dif"],"plus":[null,"Plus"]},"Job":{"version":3,"options":{},"steering":[null,"Steering"],"tasks":[[],"Task"],"difplus":[null,"DifPlus"],"description":"","categories":[[],""]},"Module":{"running_class":"","src":"","args":"","env_shell":"","env_clear":true,"configs":{},"name":"","resources":[[],"Resource"],"data":[[],"Data"],"classes":[[],"Class"],"parameters":{}},"Personnel":{"role":null,"first_name":null,"last_name":null,"email":null},"Plus":{"start":null,"end":null,"category":null,"subcategory":null,"run_number":null,"i3db_key":null,"simdb_key":null,"project":[[],{}],"steering_file":null,"log_file":null,"command_line":null},"Requirement":{"cpu":null,"gpu":null,"memory":null,"disk":null,"os":null,"site":null},"Resource":{"arch":"","remote":"","local":"","compression":false,"transfer":true},"Steering":{"parameters":{},"batchsys":[null,"Batchsys"],"system":{},"resources":[[],"Resource"],"data":[[],"Data"]},"Task":{"depends":[[],""],"batchsys":[null,"Batchsys"],"trays":[[],"Tray"],"requirements":[{"cpu":null,"gpu":null,"memory":null,"disk":null,"os":null,"site":null},"Requirement"],"task_files":false,"name":"","resources":[[],"Resource"],"data":[[],"Data"],"classes":[[],"Class"],"parameters":{}},"Tray":{"iterations":1,"modules":[[],"Module"],"name":"","resources":[[],"Resource"],"data":[[],"Data"],"classes":[[],"Class"],"parameters":{}}},"names":{"Option":"Options","Category":"Categories","Parameter":"Parameters","System":"System","Dependence":"Depends","Batchsys":"Batchsys","Class":"Classes","Data":"Data","DataCenter":"DataCenter","Dif":"Dif","DifPlus":"DifPlus","Job":"Jobs","Module":"Modules","Personnel":"Personnel","Plus":"Plus","Requirement":"Requirements","Resource":"Resources","Steering":"Steering","Task":"Tasks","Tray":"Trays"}};