@import 'theme.css';

.wy-menu-vertical li.toctree-l5 {
    font-size: 0.86em;
}
.wy-menu-vertical li.toctree-l5 > a {
    display: block;
    padding: 0.4045em 7em;
    background: #bdbdbd;
}
.wy-menu-vertical li.toctree-l5.current > a {
    color: #404040;
    border: none;
}
.wy-menu-vertical li.toctree-l6 {
    font-size: 0.82em;
}
.wy-menu-vertical li.toctree-l6 > a {
    display: block;
    padding: 0.4045em 8.25em;
    background: #bdbdbd;
}
.wy-menu-vertical li.toctree-l6.current > a {
    color: #404040;
    border: none;
}
.wy-menu-vertical li.toctree-l7 {
    font-size: 0.78em;
}
.wy-menu-vertical li.toctree-l7 > a {
    display: block;
    padding: 0.4045em 9.5em;
    background: #bdbdbd;
}
.wy-menu-vertical li.toctree-l7.current > a {
    color: #404040;
    border: none;
}

.wy-body-for-nav pre.literal-block {
    margin: 0;
    margin-bottom: 24px;
    padding: 12px 12px;
    font-family: Consolas,"Andale Mono WT","Andale Mono","Lucida Console","Lucida Sans Typewriter","DejaVu Sans Mono","Bitstream Vera Sans Mono","Liberation Mono","Nimbus Mono L",Monaco,"Courier New",Courier,monospace;
    font-size: 12px;
    line-height: 1.5;
    display: block;
    overflow: auto;
    color: #404040;
}

div.admonition.icecube p.admonition-title::before {
    content: url(../icecube_note.png);
    margin-right: 8px;
}