"""The IceProd Grid Middleware and tools."""

from __future__ import absolute_import, division, print_function

# NOTE: `__version__` is not defined because this package is built using 'setuptools-scm' --
#   use `importlib.metadata.version(...)` if you need to access version info at runtime.
